Attribute VB_Name = "bApplicationErrors"
Option Explicit

' This module is for errors specific to the application.
' Common Errors are handled in bErrorHandler.bas.

Public Enum APPLICATION_ERROR
  ' Run Time Errors
  ERR_BAD_VERSION = 513 + ERROR_NUMBER_BASE
  ERR_NO_OPEN_DATABASE
  ERR_FILE_IN_USE
  ERR_SEARCH_FIELD_NOT_ENTERED
  ERR_COMPARISON_OPERATOR_NOT_ENTERED
  ERR_SEARCH_VALUE_NOT_ENTERED
  ERR_RECORD_NOT_FOUND
  
  ' Enter additional Run Time Error Codes here

  ' Validation errors
  ERR_FIELD_NAME_REQUIRED
  ERR_DUPLICATE_FIELD_NAME
  ERR_DUPLICATE_DATA_NOT_ALLOWED
  ERR_REQUIRED_DATA_NOT_ENTERED
  ERR_DATA_BOOLEAN_OUT_OF_RANGE
  ERR_DATA_BYTE_OUT_OF_RANGE
  ERR_DATA_CURRENCY_OUT_OF_RANGE
  ERR_DATA_DATE_OUT_OF_RANGE
  ERR_DATA_INTEGER_OUT_OF_RANGE
  ERR_DATA_LONG_INTEGER_OUT_OF_RANGE
  ERR_DATA_SINGLE_PRECISION_OUT_OF_RANGE
  ERR_DATA_DOUBLE_PRECISION_OUT_OF_RANGE

  ' Enter additional Validation Error Codes here

End Enum

Public Function ShowApplicationError(nErr As Long, FileOrObject As Variant)
Dim sSearchMsg As String
Dim sObject As String

sSearchMsg = "Enter your search in the format " & DBL_QUOTE & "FIELD OPERATOR VALUE" & DBL_QUOTE & vbCrLf & "Example: UserName = Smith" & DBL_RETURN & "Valid operators include:" & vbCrLf & "=" & vbCrLf & "<>" & vbCrLf & ">" & vbCrLf & "<" & vbCrLf & ">=" & vbCrLf & "<="
sObject = CStr(FileOrObject)

' Don't log user input validation errors
ApplicationDefinedError.LogError = False

Select Case nErr ' For each error, set ApplicationDefinedError.LogError = True if you want the error to be logged

  Case ERR_BAD_VERSION
    ApplicationDefinedError.LogError = True
    ApplicationDefinedError.ErrMessage = "File is not the correct version."
    ApplicationDefinedError.DialogType = vbCritical

  Case ERR_NO_OPEN_DATABASE
    ApplicationDefinedError.ErrMessage = "No open database to save data in.  Record has not been saved."
    ApplicationDefinedError.DialogType = vbCritical

  Case ERR_FILE_IN_USE
    ApplicationDefinedError.ErrMessage = "File is already open."
    ApplicationDefinedError.DialogType = vbCritical

  Case ERR_FIELD_NAME_REQUIRED
    ApplicationDefinedError.ErrMessage = FileOrObject & vbCrLf & "Can not delete field name.  Please enter a name and try again."
    ApplicationDefinedError.DialogType = vbCritical

  Case ERR_DUPLICATE_FIELD_NAME
    ApplicationDefinedError.ErrMessage = FileOrObject & vbCrLf & "Name is already used.  Please enter another name and try again."
    ApplicationDefinedError.DialogType = vbCritical

  Case ERR_DUPLICATE_DATA_NOT_ALLOWED
    ApplicationDefinedError.ErrMessage = FileOrObject & vbCrLf & "can not contain duplicate values.  Please enter a different value and try again."
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_REQUIRED_DATA_NOT_ENTERED
    ApplicationDefinedError.ErrMessage = sObject & vbCrLf & "Required data is missing.  Record has not been saved."
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_DATA_BOOLEAN_OUT_OF_RANGE
    ApplicationDefinedError.ErrMessage = sObject & vbCrLf & "The value entered is not of the Boolean (True/False; Yes/No) data type."
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_DATA_BYTE_OUT_OF_RANGE
    ApplicationDefinedError.ErrMessage = sObject & vbCrLf & "The value entered is either not numeric or is out of range of the Byte data type.  Valid numbers are from 0 to 255."
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_DATA_CURRENCY_OUT_OF_RANGE
    ApplicationDefinedError.ErrMessage = sObject & vbCrLf & "The value entered is either not numeric or is out of range of the Currency data type.  Valid numbers are from -922,337,203,685,477.5808 to 922,337,203,685,477.5807."
    ApplicationDefinedError.DialogType = vbInformation
  
  Case ERR_DATA_DATE_OUT_OF_RANGE
    ApplicationDefinedError.ErrMessage = sObject & vbCrLf & "The value entered is either not a date or is out of range of the Date data type.  Valid dates are from January 1, 100 to December 31, 9999."
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_DATA_INTEGER_OUT_OF_RANGE
    ApplicationDefinedError.ErrMessage = sObject & vbCrLf & "The value entered is either not numeric or is out of range of the Integer data type.  Valid numbers are from -32,768 to 32,767."
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_DATA_LONG_INTEGER_OUT_OF_RANGE
    ApplicationDefinedError.ErrMessage = sObject & vbCrLf & "The value entered is either not numeric or is out of range of the Long Integer data type.  Valid numbers are from -2,147,483,648 to 2,147,483,647."
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_DATA_SINGLE_PRECISION_OUT_OF_RANGE
    ApplicationDefinedError.ErrMessage = sObject & vbCrLf & "The value entered is either not numeric or is out of range of the Single Precision numeric data type.  Valid numbers are from -3.402823E38 to -1.401298E-45 for negative values; 1.401298E-45 to 3.402823E38 for positive values."
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_DATA_DOUBLE_PRECISION_OUT_OF_RANGE
    ApplicationDefinedError.ErrMessage = sObject & vbCrLf & "The value entered is either not numeric or is out of range of the Double Precision numerica data type.  Valid numbers are from -1.79769313486231E308 to -4.94065645841247E-324 for negative values; 4.94065645841247E-324 to 1.79769313486232E308 for positive values."
    ApplicationDefinedError.DialogType = vbInformation
  
  Case ERR_SEARCH_FIELD_NOT_ENTERED
    ApplicationDefinedError.ErrMessage = "The search field you have entered was not found or is not supported for searching.  Array fields can not be searched." & DBL_RETURN & sSearchMsg
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_COMPARISON_OPERATOR_NOT_ENTERED
    ApplicationDefinedError.ErrMessage = "The search string you have entered does not contain a comparison operator.  " & sSearchMsg
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_SEARCH_VALUE_NOT_ENTERED
    ApplicationDefinedError.ErrMessage = "The search string you have entered does not contain a a value to search for.  " & sSearchMsg
    ApplicationDefinedError.DialogType = vbInformation

  Case ERR_RECORD_NOT_FOUND
    ApplicationDefinedError.ErrMessage = "No records were found matching your search."
    ApplicationDefinedError.DialogType = vbInformation

End Select

End Function
