Attribute VB_Name = "bFileHandler"
Option Explicit

Private Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function GetTempFileNameA Lib "kernel32" (ByVal lpszPath As String, ByVal lpPrefixString As String, ByVal wUnique As Long, ByVal lpTempFileName As String) As Long
Private Declare Function GetTempPathA Lib "kernel32" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
Private Declare Function lclose Lib "kernel32" Alias "_lclose" (ByVal hFile As Long) As Long
Private Declare Function lopen Lib "kernel32" Alias "_lopen" (ByVal lpPathName As String, ByVal iReadWrite As Long) As Long
Private Declare Function PathCompactPath Lib "shlwapi" Alias "PathCompactPathA" (ByVal hdc As Long, ByVal lpszPath As String, ByVal dx As Long) As Long
Private Declare Function SetFileAttributes Lib "kernel32" Alias "SetFileAttributesA" (ByVal lpFileName As String, ByVal dwFileAttributes As Long) As Long
Private Declare Function SetFileTime Lib "kernel32" (ByVal hFile As Long, lpCreationTime As tFileTime, lpLastAccessTime As tFileTime, lpLastWriteTime As tFileTime) As Long
Private Declare Function SHFileOperation Lib "shell32.dll" Alias "SHFileOperationA" (lpFileOp As SHFILEOPSTRUCT) As Long

Private Const FOF_ALLOWUNDO As Long = &H40
Private Const FO_DELETE As Long = &H3
Private Const FOF_SILENT As Long = &H4
Private Const FOF_NOCONFIRMATION As Long = &H10
Private Const OF_WRITE = &H1
Private Const OF_SHARE_DENY_WRITE = &H20
Private Const UNIQUE_NAME As Long = &H0

Public Enum OPEN_SAVE_DIALOG
  FILE_OPEN = 1
  FILE_SAVE = 2
End Enum

' Common dialog filters
Public Const ALL_FILES_FILTER As String = "All Files (*.*)|*.*|"

' ***** MODIFY *****

' Change name of extension to something unique for your app

Public Const VDB_FILES_FILTER As String = "VB DB Data Files (*.vdb)|*.vdb|"
Public Const DEFAULT_EXTENSION As String = "*.vdb"

' ***** END MODIFY *****

Private Type tFileTime
  dwLowDateTime As Long
  dwHighDateTime As Long
End Type

Private Type SHFILEOPSTRUCT
  hwnd As Long
  wFunc As Long
  pFrom As String
  pTo As String
  fFlags As Integer
  fAborted As Boolean
  hNameMaps As Long
  sProgress As String
End Type

Private Type WIN32_FIND_DATA
  dwFileAttributes As Long
  ftCreationTime As tFileTime
  ftLastAccessTime As tFileTime
  ftLastWriteTime As tFileTime
  nFileSizeHigh As Long
  nFileSizeLow As Long
  dwReserved0 As Long
  dwReserved1 As Long
  cFileName As String * MAX_PATH
  cAlternateFileName As String * 14
End Type

Public Function BuildPath(sNewPath As String) As Long
Dim FSO As New FileSystemObject
Dim nPos As Long
Dim sCurrentPath As String

On Error GoTo errHandler ' Returns Error Code

If Trim$(sNewPath) = vbNullString Then Err.Raise ERR_PATH_FILE_ACCESS

nPos = 0

Do While Not FSO.FolderExists(sNewPath)
  nPos = InStr(nPos + 1, sNewPath, BACKSLASH, vbTextCompare)
  ' Directory separator not found so use full path name
  If nPos = 0 Then nPos = Len(sNewPath)
  sCurrentPath = Left$(sNewPath, nPos)
  If Not FSO.FolderExists(sCurrentPath) Then FSO.CreateFolder sCurrentPath
Loop

Set FSO = Nothing

' Return 0

Exit Function

errHandler:
Dim nErrReturn As Long
Dim nErr As Long

nErr = Err

nErrReturn = ErrorHandler(Error, nErr, "Application folder: " & APP_FOLDER & vbCrLf & "Building path: " & sNewPath, "bFileHandler.BuildPath()")
If nErrReturn = vbRetry Then Resume

BuildPath = nErr
Set FSO = Nothing

End Function
Public Function CanWrite(sPath As String) As Long
Dim sFilename As String
Dim iFilenum As Integer
Dim nCount As Long

On Error GoTo errHandler ' Returns Error Code

' Checks if the files can be written to location specified by creating a temporary file

' Get a non-existent filename

nCount = 1

Do
  sFilename = sPath & "~tmp" & Trim$(CStr(nCount)) & ".tmp"
  nCount = nCount + 1
Loop Until Not FileExists(sFilename)

' Open the temporary file
iFilenum = FreeFile
Open sFilename For Output As #iFilenum
Close #iFilenum

' Delete the temporary file
Kill sFilename

' Return 0

Exit Function

errHandler:
CanWrite = Err
Close

End Function
Public Function CompactedPath(ByVal sPath As String, ByVal lMaxPixels As Long, ByVal hdc As Long) As String
Dim nR As Long
Dim nPos As Long

On Error GoTo errHandler

nR = PathCompactPath(hdc, sPath, lMaxPixels)
nPos = InStr(sPath, Chr$(0))

If nPos <> 0 Then
  CompactedPath = Left$(sPath, nPos - 1)
Else
  CompactedPath = sPath
End If

Exit Function

errHandler:
CompactedPath = sPath
LogError Error, Err, sPath, "bFileHandler.CompactedPath()"

End Function
Public Function CreateFolder(ByVal sFolder As String, fPromptUser As Boolean) As Long
Dim nReturn As Long
Dim FSO As New FileSystemObject

' Function should be called any time a user is prompted for a filename
' to save to.  If the folder exists, the functions does nothing and exits

On Error GoTo errHandler ' Returns Error Code

If FSO.FolderExists(sFolder) Then Exit Function

If Not fPromptUser Then
  CreateFolder = BuildPath(sFolder)
  If CreateFolder <> 0 Then Exit Function
Else
  nReturn = MsgBox("The folder " & sFolder & " does not exist.  Do you want to create it?", vbYesNoCancel + vbQuestion, APP_TITLE)
  Select Case nReturn
    Case Is = vbYes
      CreateFolder = BuildPath(sFolder)
      If CreateFolder <> 0 Then Exit Function
    Case Else
      CreateFolder = -1 ' Return if user clicks cancel
      Exit Function
  End Select
End If

Set FSO = Nothing

' Return 0

Exit Function

errHandler:
Dim nErrReturn As Long
Dim nErr As Long

nErr = Err

nErrReturn = ErrorHandler(Error, nErr, sFolder, "bFileHandler.CreateFolder()")
If nErrReturn = vbRetry Then Resume

CreateFolder = nErr
Set FSO = Nothing

End Function
Public Function GetFolder(sFilename As String) As String
Dim FSO As New FileSystemObject

On Error GoTo errHandler

' Returns the path without the filename
GetFolder = NormalizePath(FSO.GetParentFolderName(sFilename))

Set FSO = Nothing

Exit Function

errHandler:
GetFolder = vbNullString
LogError Error, Err, sFilename, "bFileHandler.GetFolder()"
Set FSO = Nothing

End Function
Public Function FileExists(ByRef sFilename As String) As Boolean
Dim FSO As New FileSystemObject

On Error GoTo errHandler

sFilename = Trim$(sFilename)
If sFilename = vbNullString Then Exit Function
FileExists = FSO.FileExists(sFilename)

Set FSO = Nothing

Exit Function

errHandler:
FileExists = False
Set FSO = Nothing

End Function
Public Function FolderExists(sPath As String) As Boolean
Dim FSO As New FileSystemObject

On Error GoTo errHandler

sPath = Trim$(sPath)
If sPath = vbNullString Then Exit Function
FolderExists = FSO.FolderExists(sPath)

Set FSO = Nothing

Exit Function

errHandler:
FolderExists = False
Set FSO = Nothing

End Function
Function GetFileName(OpenOrSave As OPEN_SAVE_DIALOG, InitFilename As String, Flags As Long, Filters As String, InitFolder As String, CurrentFilename As String, Optional DefaultExtension As String) As String
Dim sFilename As String
Dim nReturn As Long
Dim FSO As New FileSystemObject

On Error GoTo errHandler

Top:

With CommonDialog
  .CancelError = True
  .Filename = InitFilename
  .Flags = Flags Or cdlOFNHideReadOnly
  .Filter = Filters
  .InitDir = InitFolder

  Select Case OpenOrSave
    Case FILE_OPEN
      .Flags = .Flags Or cdlOFNFileMustExist
      If Not IsMissing(DefaultExtension) Then .DefaultExt = DefaultExtension
      .ShowOpen
      sFilename = .Filename
      If sFilename = vbNullString Then Exit Function

    Case FILE_SAVE
      .ShowSave
      sFilename = Trim$(.Filename)
      If sFilename = vbNullString Then Exit Function

      ' This case occurs only when a user tries to save the currently open file
      ' to itself using commands such as "save as" or "backup" etc.
      If FileExists(CurrentFilename) And (StrComp(CurrentFilename, sFilename, vbTextCompare) = 0) Then
        MsgBox "Can't save file to itself.  Please choose a different file name to save as.", vbCritical, APP_TITLE
        GoTo Top
      End If

      If FileExists(sFilename) Then
        nReturn = MsgBox("Overwrite existing " & sFilename & "?", vbYesNoCancel + vbQuestion, APP_TITLE)
        Select Case nReturn
          Case Is = vbYes
            ShellDelete sFilename ' Sends file to recycle bin
          Case Is = vbNo
            GoTo Top ' Try again
          Case Is = vbCancel
            Exit Function
        End Select
      End If

      nReturn = CreateFolder(GetFolder(sFilename), True)
      If nReturn <> 0 Then Err.Raise nReturn

      ' Raise error if file can't be created
      nReturn = CanWrite(GetFolder(sFilename))
      If nReturn <> 0 Then Err.Raise nReturn
    End Select
End With

GetFileName = sFilename

Set FSO = Nothing

Exit Function

errHandler:
Dim nErrReturn As Long

If Err = ERR_CANCEL_WAS_SELECTED Then Exit Function

nErrReturn = ErrorHandler(Error, Err, CommonDialog.Filename, "bFileHandler.GetFileName()")
If nErrReturn = vbRetry Then Resume Top

GetFileName = vbNullString

Set FSO = Nothing

End Function
Public Function GetTempFilename() As String
Dim sString As String
Dim sString1 As String
Dim sString2 As String

On Error GoTo errHandler

sString2 = GetTempPath
sString1 = Space(Len(sString2) + MAX_PATH)

Call GetTempFileNameA(sString2, "~", UNIQUE_NAME, sString1)
sString = TrimCharacter(sString1, vbNullChar)
GetTempFilename = sString

Exit Function

errHandler:
Dim nErrReturn As Long

nErrReturn = ErrorHandler(Error, Err, sString, "bFileHandler.GetTempFilename()")
If nErrReturn = vbRetry Then Resume

GetTempFilename = vbNullString

End Function
Public Function GetTempPath() As String
Dim iReturn As Integer
Dim sString As String

On Error GoTo errHandler

iReturn = GetTempPathA(0, vbNullString)
sString = Space$(iReturn)

Call GetTempPathA(iReturn, sString)
GetTempPath = NormalizePath(TrimCharacter(sString, vbNullChar))

Exit Function

errHandler:
LogError Error, Err, vbNullString, "bFileHandler.GetTempPath()"
GetTempPath = APP_FOLDER

End Function
Public Function FileLastModified(sFilename As String) As Date
Dim FSO As New FileSystemObject
Dim m_File As File

If Not FileExists(sFilename) Then Exit Function

Set m_File = FSO.GetFile(sFilename)
FileLastModified = m_File.DateLastModified

Set FSO = Nothing

End Function
Public Function NormalizePath(sFolder As String) As String

sFolder = Trim$(sFolder)

If Right$(sFolder, 1) <> BACKSLASH Then sFolder = sFolder & BACKSLASH
NormalizePath = sFolder

End Function
Function ReplaceFile(sOldFile As String, sTempFile As String) As Boolean
Dim fnd As WIN32_FIND_DATA
Dim nOld As Long
Dim f As Boolean

' Get file time and attributes of old file
If FindFirstFile(sOldFile, fnd) = -1 Then Exit Function

' Replace by deleting old and renaming new to old

Kill sOldFile
Name sTempFile As sOldFile

On Error GoTo 0

' Assign old attributes and time to new file
nOld = lopen(sOldFile, OF_WRITE Or OF_SHARE_DENY_WRITE)
f = SetFileTime(nOld, fnd.ftCreationTime, fnd.ftLastAccessTime, fnd.ftLastWriteTime)
lclose nOld

If Not f Then Exit Function

f = SetFileAttributes(sOldFile, fnd.dwFileAttributes)

If f Then ReplaceFile = True

End Function
Public Function ShellDelete(ParamArray sFilename() As Variant) As Boolean
Dim iReturn As Integer
Dim sFileNames As String
Dim SHFileOp As SHFILEOPSTRUCT

' Sends files to Recycle Bin

On Error GoTo errHandler

For iReturn = LBound(sFilename) To UBound(sFilename)
  sFileNames = sFileNames & sFilename(iReturn) & vbNullChar
Next iReturn

sFileNames = sFileNames & vbNullChar

With SHFileOp
  .wFunc = FO_DELETE
  .pFrom = sFileNames
  .fFlags = FOF_ALLOWUNDO + FOF_SILENT + FOF_NOCONFIRMATION
End With

Top:
iReturn = SHFileOperation(SHFileOp)
If iReturn <> 0 Then Err.Raise iReturn

Exit Function

errHandler:
Dim nErrReturn As Long

nErrReturn = ErrorHandler(Error, Err, Join(sFilename, ", "), "bFileHandler.ShellDelete()")
If nErrReturn = vbRetry Then Resume Top

End Function

