Attribute VB_Name = "bFunctions"
Option Explicit

Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long, ByVal wNewLong As Long) As Long
Public Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" (ByVal hwnd As Long, ByVal lpOperation As String, ByVal lpFile As String, ByVal lpParameters As String, ByVal lpDirectory As String, ByVal nShowCmd As Long) As Long

Private Const GWW_HWNDPARENT = -8

Public Function AlwaysOnTop(frm As Form, frmParent As Form) As Long

On Error Resume Next

AlwaysOnTop = SetWindowLong(frm.hwnd, GWW_HWNDPARENT, frmParent.hwnd)

End Function
Public Function CanCopy(ctl As Control) As Boolean

On Error GoTo errHandler

' CanCopy = False

If Not ctl.Enabled Then Exit Function
If Not IsTextControl(ctl) Then Exit Function
If Not TextSelected(ctl) Then Exit Function

CanCopy = True

Exit Function

errHandler:
LogError Error, Err, vbNullString, "bFunctions.CanCopy()"

End Function
Public Function CanCut(ctl As Control) As Boolean

On Error GoTo errHandler

' CanCut = False

If Not ctl.Enabled Then Exit Function
If Not IsTextControl(ctl) Then Exit Function
If Not TextSelected(ctl) Then Exit Function

If TypeName(ctl) = "TextBox" Or TypeName(ctl) = "RichTextBox" Then
  If ctl.Locked Then Exit Function
End If

CanCut = True

Exit Function

errHandler:
LogError Error, Err, vbNullString, "bFunctions.CanCut()"

End Function
Public Function CanPaste(ctl As Control) As Boolean

On Error GoTo errHandler

' CanPaste = False

If Not ctl.Enabled Then Exit Function
If Clipboard.GetText = vbNullString Then Exit Function
If Not IsTextControl(ctl) Then Exit Function

CanPaste = True

Exit Function

errHandler:
LogError Error, Err, vbNullString, "bFunctions.CanPaste()"

End Function
Public Function CheckNumeric(sString As String, AllowDecimal As Boolean) As Boolean
Dim i As Integer

' Determines if a string is a numeric value

For i = 1 To Len(sString)
  Select Case Mid$(sString, i, 1)

    Case "0" To "9"
      ' Do nothing

    Case "-", "+"
      ' Minus/plus signs are only allowed as leading chars.
      If i > 1 Then Exit Function

    Case "."
      ' Exit if decimal values not allowed.
      If Not AllowDecimal Then Exit Function
      ' Only one decimal separator is allowed.
      If InStr(sString, ".") < i Then Exit Function

    Case Else
    ' Reject all other characters.
    Exit Function

  End Select
Next i

CheckNumeric = True

End Function
Public Sub Hourglass(fSetHourglass As Boolean)

' Changes the MousePointer to an Hourglass or to Normal depending on what is passed.

On Error Resume Next

Screen.MousePointer = IIf(fSetHourglass, vbHourglass, vbDefault)

End Sub
Private Function IsList(lst As Control) As Boolean

' Determines if control is a list

IsList = (TypeName(lst) = "ListBox") Or (TypeName(lst) = "ComboBox")

End Function
Public Function IsTextControl(ctl As Control)

If TypeName(ctl) = "TextBox" Or TypeName(ctl) = "MaskEdBox" Or TypeName(ctl) = "RichTextBox" Then
  IsTextControl = True
End If

End Function
Public Function ListIndexFromItemData(lst As Control, iItemData As Long) As Long
Dim n As Long

' Returns the Listindex

On Error GoTo errHandler

ListIndexFromItemData = -1

If Not IsList(lst) Then Exit Function

With lst
  For n = 0 To .ListCount - 1
    If .ItemData(n) = iItemData Then
      ListIndexFromItemData = n
      Exit Function
    End If
  Next n
End With

Exit Function

errHandler:
LogError Error, Err, lst.Name, "bFunctions.ListIndexFromItemData()"

End Function
Public Function RegistrySettingExists(sSection As String, sKey As String) As Boolean
Dim s

On Error GoTo errHandler

s = GetSetting(APP_TITLE, sSection, sKey, vbNullString)
RegistrySettingExists = (s <> vbNullString)

Exit Function

errHandler:
RegistrySettingExists = False

End Function
Public Sub RestoreWindowPosition(Form As Form, ParentForm As Form)
Dim nLeft As Long
Dim nTop As Long
Dim nDefaultLeft As Long
Dim nDefaultTop As Long

' Retrieves window position from registry if entered
' If no entry, form is centered on parent.
' If window is off the screen it is moved back onto screen @(0, 0)

On Error GoTo errHandler

With Form
  nDefaultLeft = (ParentForm.Width - .Width) / 2
  nDefaultTop = (ParentForm.Height - .Height) / 2
  nLeft = GetSetting(APP_TITLE, "Windows", .Name & " left", nDefaultLeft)
  nTop = GetSetting(APP_TITLE, "Windows", .Name & " top", nDefaultTop)
  Select Case nLeft
    Case Is < 0
      nLeft = 0
    Case Is >= (Screen.Width - .Width)
      nLeft = Screen.Width - .Width
  End Select
  Select Case nTop
    Case Is < 0
      nTop = 0
    Case Is >= (Screen.Height - .Height)
      nTop = Screen.Height - .Height
  End Select
  .Left = nLeft
  .Top = nTop
End With

Exit Sub

errHandler:
LogError Error, Err, vbNullString, "bFunctions.RestoreWindowPosition()"

End Sub
Public Function SaveWindowPosition(frm As Form)

' Saves last position of window to the registry

With frm
  SaveSetting APP_TITLE, "Windows", .Name & " left", .Left
  SaveSetting APP_TITLE, "Windows", .Name & " top", .Top
End With

End Function
Public Sub SelectAllText(ctl As Control)

' Selects all text in a TextBox or MaskEdBox
' Most commonly used in GotFocus event

On Error GoTo errHandler

If ctl Is Nothing Then Exit Sub

With ctl
  If .Parent.Visible = False Or .Visible = False Or .Enabled = False Then Exit Sub
  If TypeName(ctl) = "TextBox" Or TypeName(ctl) = "MaskEdBox" Then
    .SelStart = 0
    .SelLength = Len(.Text)
  End If
End With

Exit Sub

errHandler:
LogError Error, Err, vbNullString, "bFunctions.SelectAllText()"

End Sub
Public Function TextSelected(ctl As Control) As Boolean

' Determines if text is selected.

On Error GoTo errHandler

TextSelected = Len(ctl.SelText) > 0

Exit Function

errHandler:
TextSelected = False

End Function
Public Function TrimCharacter(ByRef s As String, Character As String) As String
Dim iLen As Integer

' Loops through a string a trims all instances of a character from both ends.

On Error GoTo errHandler

iLen = Len(Character)

s = Trim$(s)

Do While Left$(s, iLen) = Character
  s = Right$(s, Len(s) - iLen)
Loop

Do While Right$(s, iLen) = Character
  s = Left$(s, Len(s) - iLen)
Loop

TrimCharacter = s

Exit Function

errHandler:
TrimCharacter = s

End Function

