VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cMRU"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'---------------------------------------------------------------------------------------
' Module    : cMRU
' DateTime  : 6/3/2002 00:23
' Author    : Paul K. Johnson
' Purpose   : Manipulates Most Recently Used File menu items and stores data in registry
'---------------------------------------------------------------------------------------
Option Explicit

' Dependencies
' cMRU_File

' Notes:

' * Create a new instance of this class in the Form Declarations.
' * Set the Form property in the Form_Load event.
' * Set Max_MRU_Count in the form_load event or use the default of 4 (class initialize).
' * The form must have a mnuFileMRU menu array or an error will occur.
' * This class takes care of all the MRU menu processing and saving to/restoring from the registry.
' * The application must handle all file processing - i.e. opening the selected file
' * When the form is passed, this class reads the registry and loads the menus or makes them invisible as appropriate.
' * The full pathname of the file is saved to the mnuFileMRU(Index).tag property.
' * Add a menu item above the mnuFileMRU item named mnuFileMRUSeparator

' Example use:

' =================================
'
' Form Declarations section
' Private MRU as new cMRU
'
' =================================
' Sub Form_Load
'
'   Set MRU.Form = Me
'   MRU.MAX_MRU_COUNT = 4
'
' End sub
'
' =================================
'
' Sub mnuFileMRU(Index)_Click
'   Dim iReturn as long
'
'   On Error Goto ErrHandler
'   ' Add your procedure for processing the file as applies to your application:
'   iReturn = ProcessFile (mnuFileMRU(Index).tag
'   if iReturn <> 0 then Err.Raise iReturn
'
'   MRU.Add mnuFileMRU(Index).tag
'
'   Exit sub
'
'   ErrHandler:
'
'   ' Permanently remove the file from the MRU list
'   MRU.Remove mnuFileMRU(Index).Tag
'
' End Sub
'
' =================================

' Public properties

Public Max_MRU_Count As Integer

' Private properties
Private colMRU As New Collection
Private frm As Form

Public Sub Add(ByRef Filename As String)

Filename = Trim$(Filename)

' Refresh MRU list
ReadRegistry

' Remove the file from the collection so it can be added at the top of the stack
If RemoveInternal(Filename) <> 0 Then Exit Sub

' Add to collection
AddInternal Filename

' Save to registry
SaveRegistry

' Refresh the form's menus
PopulateMRUMenu

End Sub
Private Sub AddInternal(Filename As String)
Dim i As Integer
Dim MRU_File As New cMRU_File

On Error GoTo errHandler

Filename = Trim$(Filename)

If Filename = vbNullString Then Exit Sub
If KeyExists(Filename) Then Exit Sub

With MRU_File
  .Form = frm
  .Filename = Filename
End With

' Add the file to the collection
With colMRU
  If .Count > 0 Then
    .Add MRU_File, MRU_File.Filename, 1 ' Add to the top of the collection
  Else
    .Add MRU_File, MRU_File.Filename    ' Just add it to the collection
  End If
  For i = Max_MRU_Count + 1 To .Count   ' Remove excess items if necessary
    .Remove (i)
  Next i
End With

Exit Sub

errHandler:
LogError Error, Err, Filename, "cMRU.AddInternal()"
Resume Next

End Sub
Public Property Set Form(Form As Form)

Set frm = Form

' Read registry when property is set to retrieve MRU items
ReadRegistry

End Property
Private Function KeyExists(Filename As String) As Boolean
Dim i As Integer

On Error GoTo errHandler

For i = 1 To colMRU.Count
  If StrComp(colMRU(i).Filename, Filename, vbTextCompare) = 0 Then
    KeyExists = True
    Exit Function
  End If
Next i

Exit Function

errHandler:
LogError Error, Err, Filename, "cMRU.KeyExists()"

End Function
Private Sub PopulateMRUMenu()
Dim n As Long
Dim iCount As Long

On Error GoTo errHandler

If frm Is Nothing Then Exit Sub

If colMRU.Count = 0 Then frm.mnuFileMRU(0).Visible = False

With frm
  iCount = .mnuFileMRU.Count - 1

  For n = iCount To 1 Step -1
    Unload .mnuFileMRU(n)
  Next n

  For n = colMRU.Count - 1 To 1 Step -1
    Load .mnuFileMRU(n)
  Next n

  For n = 0 To colMRU.Count - 1
    .mnuFileMRU(n).Caption = colMRU(n + 1).ShortFilename
    .mnuFileMRU(n).Tag = colMRU(n + 1).Filename
    .mnuFileMRU(n).Visible = True
  Next n
  .mnuFileMRUSeparator.Visible = .mnuFileMRU(0).Visible
End With

Exit Sub

errHandler:
LogError Error, Err, vbNullString, "cMRU.PopulateMRUMenu()"

End Sub
Private Function ReadRegistry()
Dim i As Integer
Dim v As Variant
Dim sKey As String
Dim sFilename As String
Dim iLen As Integer

On Error GoTo errHandler

iLen = Len(frm.Name)

v = GetAllSettings(APP_TITLE, "MRU")
If IsEmpty(v) Then Exit Function

For i = UBound(v, 1) To LBound(v, 1) Step -1
  sKey = v(i, 0)
  If StrComp(Left(sKey, iLen), frm.Name, vbTextCompare) = 0 Then
    sFilename = v(i, 1)
    If FileExists(CStr(sFilename)) Then AddInternal sFilename
  End If
Next i

PopulateMRUMenu

Exit Function

errHandler:
LogError Error, Err, vbNullString, "cMRU.ReadRegistry()"

End Function
Public Function Remove(Filename As String) As Long

RemoveInternal Filename
PopulateMRUMenu

End Function
Private Function RemoveInternal(Filename As String)
Dim i As Integer
Dim sKey As String

On Error GoTo errHandler

For i = 1 To colMRU.Count
  If StrComp(colMRU(i).Filename, Filename, vbTextCompare) = 0 Then

    colMRU.Remove i

    sKey = frm.Name & "_MRU" & Trim$(Str$(i))
    If RegistrySettingExists("MRU", sKey) Then
      DeleteSetting APP_TITLE, "MRU", sKey
    End If

    Exit For
  
  End If
Next i

SaveRegistry

Exit Function

errHandler:
RemoveInternal = Err
LogError Error, Err, Filename, "cMRU.RemoveInternal()"

End Function
Private Function SaveRegistry()
Dim i As Integer

On Error GoTo errHandler

For i = 1 To colMRU.Count
  SaveSetting APP_TITLE, "MRU", frm.Name & "_MRU" & Trim$(Str$(i)), colMRU(i).Filename
Next i

Exit Function

errHandler:
LogError Error, Err, vbNullString, "cMRU.SaveRegistry()"

End Function
Public Property Get ShortFilename(Index As Integer)

ShortFilename = colMRU(Index + 1).ShortFilename

End Property
Private Sub Class_Initialize()

Max_MRU_Count = 4

End Sub
