VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cTable"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'---------------------------------------------------------------------------------------
' Module    : cTable
' DateTime  : 5/26/2002
' Author    : Paul K. Johnson
' Purpose   : Creates a Binary Access file for saving record-type data. Uses cRecord class
'
' This Class includes multi-user support.  It is slower than the single-user
' version, but allows multiple users to access the same file without over-writing each
' others changes.
'---------------------------------------------------------------------------------------

Option Explicit

Public RecordNumber As Long
Public SearchField As FIELD_INDEX
Public SearchOperator As COMPARISON_OPERATOR
Public SearchValue As Variant

Private Const CurrentDateTime As Integer = -1
Private BackupFilename As String

Private LastModified As Date
Private fBackup As Boolean
Private lngEditMode As Long
Private sFilename As String

' Additional fields can be added to the Header.
' DO NOT REMOVE OR RENAME these Four fields!

' VersionString
' VersionNumber
' RecordCount
' MaxRecordID

Private Type tHeader
  VersionString As String
  VersionNumber As String
  RecordCount As Long
  MaxRecordID As Long ' The maximum Index of all records
End Type
Private Header As tHeader

Private Type tFieldDefinition
  Index As Long
  SystemField As Boolean
  FieldName As String
  ArrayField As Boolean
  DataType As Long
  DefaultValue As Variant
  Required As Boolean
  RequireUniqueEntry As Boolean
End Type

Private m_FieldDefinitions(FIELD_INDEX_MAX) As tFieldDefinition

Private Type tRecord
  Field(FIELD_INDEX_MAX) As Variant
End Type

Private arr_Record() As tRecord

Public Function AddRecord() As Long

If EditMode = ADD_NEW Then Exit Function

EditMode = ADD_NEW
RecordNumber = RecordCount + 1

End Function
Private Sub CreateDataFields()

' ***** MODIFY *****

' Before creating fields, modify the FIELD_INDEX Enum in bVBDB.bas
' Then use the values from the Enum as subscripts for your fields

' Note: Assign the Constant CurrentDateTime (-1) to a date
' field to have the value of Now() entered into the value field
' when a new record is created.

' Generic Boolean field
With m_FieldDefinitions(FIELD_01)
  .ArrayField = False
  .DataType = dataBoolean
  .DefaultValue = False
  .FieldName = "Boolean 1"
  .Index = NewFieldIndex
  .Required = False
  .RequireUniqueEntry = False
  .SystemField = False
End With

' Generic Byte field
With m_FieldDefinitions(FIELD_02)
  .ArrayField = False
  .DataType = dataByte
  .DefaultValue = 0
  .FieldName = "Byte 1"
  .Index = NewFieldIndex
  .Required = False
  .RequireUniqueEntry = False
  .SystemField = False
End With

' Generic Date field
With m_FieldDefinitions(FIELD_03)
  .ArrayField = False
  .DataType = dataDate
  .DefaultValue = CurrentDateTime
  .FieldName = "Date 1"
  .Index = NewFieldIndex
  .Required = False
  .RequireUniqueEntry = False
  .SystemField = False
End With

' Generic Double Precision array field
With m_FieldDefinitions(FIELD_04)
  .ArrayField = True
  .DataType = dataDoublePrecision
  .DefaultValue = 0
  .FieldName = "Double Array"
  .Index = NewFieldIndex
  .Required = False
  .RequireUniqueEntry = False
  .SystemField = False
End With

' Generic Integer field
With m_FieldDefinitions(FIELD_05)
  .ArrayField = False
  .DataType = dataInteger
  .DefaultValue = 0
  .FieldName = "Integer 1"
  .Index = NewFieldIndex
  .Required = False
  .RequireUniqueEntry = False
  .SystemField = False
End With

' Generic Long Integer field
With m_FieldDefinitions(FIELD_06)
  .ArrayField = False
  .DataType = dataLongInteger
  .DefaultValue = 0
  .FieldName = "Long 1"
  .Index = NewFieldIndex
  .Required = False
  .RequireUniqueEntry = False
  .SystemField = False
End With

' Generic Single Precision field
With m_FieldDefinitions(FIELD_07)
  .ArrayField = False
  .DataType = dataSinglePrecision
  .DefaultValue = 0
  .FieldName = "Single 1"
  .Index = NewFieldIndex
  .Required = False
  .RequireUniqueEntry = False
  .SystemField = False
End With

With m_FieldDefinitions(FIELD_08)
  .ArrayField = False
  .DataType = dataString
  .DefaultValue = vbNullString
  .FieldName = "String 1"
  .Index = NewFieldIndex
  .Required = False
  .RequireUniqueEntry = False
  .SystemField = False
End With

' ***** END MODIFY *****

End Sub
Private Sub CreateSystemFields()

' ****** DO NOT modify these fields! ******

With m_FieldDefinitions(RECORD_ID)
  .ArrayField = False
  .DataType = dataLongInteger
  .DefaultValue = 0
  .FieldName = "ID"
  .Index = NewFieldIndex
  .Required = True
  .RequireUniqueEntry = True
  .SystemField = True
End With

With m_FieldDefinitions(DATE_CREATED)
  .ArrayField = False
  .DataType = dataDate
  .DefaultValue = CurrentDateTime
  .FieldName = "Date Created"
  .Index = NewFieldIndex
  .Required = True
  .RequireUniqueEntry = False
  .SystemField = True
End With

With m_FieldDefinitions(LAST_MODIFIED)
  .ArrayField = False
  .DataType = dataDate
  .DefaultValue = CurrentDateTime
  .FieldName = "Last Modified"
  .Index = NewFieldIndex
  .Required = True
  .RequireUniqueEntry = False
  .SystemField = True
End With

End Sub
Private Sub CreateHeader()

With Header
  .RecordCount = RecordCount
  .VersionNumber = VBDB_VERSION_NUMBER
  .VersionString = VBDB_VERSION_STRING
End With

End Sub
Private Function CreateTemporaryBackup() As Long
Dim FSO As New FileSystemObject

On Error GoTo errHandler

' GetTempFilename call creates the file so delete it
BackupFilename = GetTempFilename
If FileExists(BackupFilename) Then Kill BackupFilename

' Save current file to backup filename
If FileExists(Filename) Then
  FSO.CopyFile Filename, BackupFilename, True
  fBackup = True
End If

Set FSO = Nothing

Exit Function

errHandler:
CreateTemporaryBackup = Err

End Function
Private Property Get DefaultRecord() As cRecord
Dim i As Long
Dim m_Record As New cRecord

' Enters default values into fields of Record object

With m_Record
  For i = 0 To FieldCount - 1
    If (m_FieldDefinitions(i).DataType = dataDate) And (m_FieldDefinitions(i).DefaultValue = CurrentDateTime) Then
      .Fields(i) = Now()
    Else
      .Fields(i) = FieldDefinitions(i, fldDefaultValue)
    End If
  Next i
  .Fields(DATE_CREATED) = Now()
  .Fields(LAST_MODIFIED) = .Fields(DATE_CREATED)
  .Fields(RECORD_ID) = 0
End With

Set DefaultRecord = m_Record
Set m_Record = Nothing

End Property
Public Function DeleteRecord() As Long
Dim i As Long
Dim lngReturn As Long
Dim iFilenum As Integer

On Error GoTo errHandler

lngReturn = InitializeFileSave(iFilenum)
If lngReturn <> 0 Then Err.Raise lngReturn

If RecordNumber = 0 Then ' Record was already deleted
  If RecordCount > 0 Then RecordNumber = 1
  Close #iFilenum
  Exit Function
End If

If RecordCount = 1 And RecordNumber = 1 Then
  ' Record is the last record in file so erase the array
  Erase arr_Record
Else
  ' Move all records after deleted record forward one position in array
  For i = RecordNumber - 1 To UBound(arr_Record) - 1
    arr_Record(i) = arr_Record(i + 1)
  Next i

  ' Redimension the array to remove the last element
  ReDim Preserve arr_Record(UBound(arr_Record) - 1)

  ' Move to previous record
  If RecordNumber > 1 Then
    RecordNumber = RecordNumber - 1
  Else
    RecordNumber = 1
  End If
End If

lngReturn = FinalizeFileSave(iFilenum)
If lngReturn <> 0 Then Err.Raise lngReturn

Exit Function

errHandler:
DeleteRecord = Err

Close #iFilenum
If fBackup Then RestoreBackupFile

End Function
Private Property Let EditMode(iMode As EDITING_MODE)

lngEditMode = iMode

End Property
Public Property Get EditMode() As EDITING_MODE

EditMode = lngEditMode

End Property
Public Property Get FieldCount() As Long
Dim t As tRecord

FieldCount = UBound(t.Field) + 1

End Property
Public Property Get FieldDefinitions(Index As FIELD_INDEX, FieldDefinition As FIELD_DEFINITION) As Variant

With m_FieldDefinitions(Index)

  Select Case FieldDefinition

    Case Is = fldArray
      FieldDefinitions = .ArrayField

    Case Is = fldDataType
      FieldDefinitions = .DataType

    Case Is = fldDefaultValue
      FieldDefinitions = .DefaultValue

    Case Is = fldIndex
      FieldDefinitions = .Index

    Case Is = fldName
      FieldDefinitions = .FieldName

    Case Is = fldRequired
      FieldDefinitions = .Required

    Case Is = fldRequireUnique
      FieldDefinitions = .RequireUniqueEntry

    Case Is = fldSystemField
      FieldDefinitions = .SystemField

  End Select

End With

End Property
Public Property Let FieldDefinitions(Index As FIELD_INDEX, FieldDefinition As FIELD_DEFINITION, Value As Variant)

With m_FieldDefinitions(Index)

  ' Changes to System fields are not allowed
  If .SystemField Then Exit Property

  Select Case FieldDefinition

    Case Is = fldArray
      ' Do nothing

    Case Is = fldDataType
      .DataType = Value

    Case Is = fldDefaultValue
      .DefaultValue = Value

    Case Is = fldName
      .FieldName = Value

    Case Is = fldRequired
      .Required = Value

    Case Is = fldRequireUnique
      .RequireUniqueEntry = Value

    Case Is = fldSystemField
      ' Do nothing

  End Select

End With

End Property
Public Property Get Filename() As String

Filename = sFilename

End Property
Public Property Let Filename(DataFilename As String)
Dim lngReturn As Long
Dim iFilenum As Integer
Dim sTmp As String

' File is read when Filename property is set.

On Error GoTo errHandler

sTmp = DataFilename

iFilenum = FreeFile

If FileExists(sTmp) Then
  ' Find out if file is a valid VB DB file
  Open sTmp For Binary Lock Read Write As #iFilenum

  lngReturn = Read(sTmp, iFilenum)
  If lngReturn <> 0 Then Err.Raise lngReturn
Else
  ' Create a new database file
  sFilename = sTmp
  Open sFilename For Binary Lock Read Write As #iFilenum

  lngReturn = WriteFile(iFilenum)
  If lngReturn <> 0 Then Err.Raise lngReturn

  AddRecord
End If

Close #iFilenum

sFilename = sTmp

LastModified = FileLastModified(sFilename)

Exit Property

errHandler:
Dim iError As Long
iError = ErrorHandler(Error, Err, sFilename, "cDatabase.Filename(Property Let)")
If iError = vbRetry Then Resume

Close #iFilenum

End Property
Private Function FinalizeFileSave(FileNumber As Integer) As Long
Dim lngReturn As Long

On Error GoTo errHandler

' Save records to file
lngReturn = WriteFile(FileNumber)
If lngReturn <> 0 Then Err.Raise lngReturn

' Close file
Close #FileNumber

LastModified = FileLastModified(Filename)

' Delete temporary backup file
If FileExists(BackupFilename) Then Kill BackupFilename

fBackup = False

Exit Function

errHandler:
FinalizeFileSave = Err

End Function
Public Function Find(Direction As SEARCH_DIRECTION) As Long
Dim i As Long
Dim lngReturn As Long

' Returns Recordnumber or 0 if no record found.

' Does not change current record number.
' Calling procedure must change the recordnumber.

On Error GoTo errHandler

Select Case Direction
  Case Is = FIND_FIRST ' Start at beginning of file and search to the end
    For i = 0 To RecordCount - 1
      lngReturn = FindCompare(arr_Record(i).Field(SearchField))
      If lngReturn = 0 Then
        Find = i + 1
        Exit Function
      End If
    Next i

  Case Is = FIND_PREVIOUS ' Start at previous record and search to beginning
    For i = RecordNumber - 2 To 0 Step -1
      lngReturn = FindCompare(arr_Record(i).Field(SearchField))
      If lngReturn = 0 Then
        Find = i + 1
        Exit Function
      End If
    Next i
 
  Case Is = FIND_NEXT ' Start at next record and search to the end
    For i = RecordNumber To RecordCount - 1
      lngReturn = FindCompare(arr_Record(i).Field(SearchField))
      If lngReturn = 0 Then
        Find = i + 1
        Exit Function
      End If
    Next i

  Case Is = FIND_LAST ' Start at last record and search to the beginning
    For i = RecordCount - 1 To 0 Step -1
      lngReturn = FindCompare(arr_Record(i).Field(SearchField))
      If lngReturn = 0 Then
        Find = i + 1
        Exit Function
      End If
    Next i

End Select

Err.Raise ERR_RECORD_NOT_FOUND

Exit Function

errHandler:
Find = 0
ErrorHandler Error, Err, SearchValue, "cTable.Find"

End Function
Private Function FindCompare(v_Field As Variant) As Long

' Returns 0 if record is found

On Error GoTo errHandler

If IsDate(v_Field) And IsDate(SearchValue) Then
  ' Date comparison
  Select Case SearchOperator
    Case EQUAL
      If CDate(v_Field) = CDate(SearchValue) Then Exit Function
    Case NOT_EQUAL
      If CDate(v_Field) <> CDate(SearchValue) Then Exit Function
    Case LESS_THAN
      If CDate(v_Field) < CDate(SearchValue) Then Exit Function
    Case GREATER_THAN
      If CDate(v_Field) > CDate(SearchValue) Then Exit Function
    Case LESS_THAN_OR_EQUAL
      If CDate(v_Field) <= CDate(SearchValue) Then Exit Function
    Case GREATER_THAN_OR_EQUAL
      If CDate(v_Field) >= CDate(SearchValue) Then Exit Function
  End Select

ElseIf CheckNumeric(CStr(v_Field), True) And CheckNumeric(CStr(SearchValue), True) Then
  ' Numeric comparison
  Select Case SearchOperator
    Case EQUAL
      If v_Field = Val(SearchValue) Then Exit Function
    Case NOT_EQUAL
      If v_Field <> Val(SearchValue) Then Exit Function
    Case LESS_THAN
      If v_Field < Val(SearchValue) Then Exit Function
    Case GREATER_THAN
      If v_Field > Val(SearchValue) Then Exit Function
    Case LESS_THAN_OR_EQUAL
      If v_Field <= Val(SearchValue) Then Exit Function
    Case GREATER_THAN_OR_EQUAL
      If v_Field >= Val(SearchValue) Then Exit Function
  End Select

Else ' Default
  ' Text comparison
  Select Case SearchOperator
    Case EQUAL ' Works similar to * LIKE *
      If InStr(1, CStr(v_Field), CStr(SearchValue), vbTextCompare) > 0 Then Exit Function
    Case NOT_EQUAL
      If InStr(1, CStr(v_Field), CStr(SearchValue), vbTextCompare) = 0 Then Exit Function
    Case LESS_THAN
      If StrComp(CStr(SearchValue), CStr(v_Field), vbTextCompare) = -1 Then Exit Function
    Case GREATER_THAN
      If StrComp(CStr(SearchValue), CStr(v_Field), vbTextCompare) = 1 Then Exit Function
    Case LESS_THAN_OR_EQUAL
      If StrComp(CStr(SearchValue), CStr(v_Field), vbTextCompare) <= 0 Then Exit Function
    Case GREATER_THAN_OR_EQUAL
      If StrComp(CStr(SearchValue), CStr(v_Field), vbTextCompare) >= 0 Then Exit Function
  End Select

End If

' Function will not run to here if search was successful
Err.Raise ERR_RECORD_NOT_FOUND

Exit Function

errHandler:
FindCompare = Err

End Function
Private Function GetRecordNumberFromID(ID As Long) As Long
Dim i As Long

If RecordCount < 1 Then Exit Function

For i = LBound(arr_Record) To UBound(arr_Record)
  If ID = arr_Record(i).Field(RECORD_ID) Then
    GetRecordNumberFromID = i + 1
    Exit For
  End If
Next i

End Function
Private Function HasDuplicateEntry(ByRef Record As cRecord) As Long
Dim i As Long
Dim j As Long

On Error GoTo errHandler

For i = 0 To RecordCount - 1
  If i <> RecordNumber - 1 Then
    For j = 0 To FieldCount - 1
      With m_FieldDefinitions(j)
        If .RequireUniqueEntry And Not .ArrayField Then
          If .DataType = dataString Then
            If StrComp(arr_Record(i).Field(j), Record.Fields(j), vbTextCompare) = 0 Then Err.Raise ERR_DUPLICATE_DATA_NOT_ALLOWED
          Else
            If arr_Record(i).Field(j) = Record.Fields(j) Then Err.Raise ERR_DUPLICATE_DATA_NOT_ALLOWED
          End If
        End If
      End With
    Next j
  End If
Next i

Exit Function

errHandler:
HasDuplicateEntry = Err
ErrorHandler Error, Err, m_FieldDefinitions(j).FieldName, "cTable.HasDuplicateEntry"

End Function
Private Function InitializeFileSave(ByRef FileNumber As Integer) As Long
Dim lngReturn As Long
Dim lngID As Long

On Error GoTo errHandler

' Save RecordID
If EditMode = EDIT_MODE Then
  lngID = arr_Record(RecordNumber - 1).Field(RECORD_ID)
End If

lngReturn = CreateTemporaryBackup
If lngReturn <> 0 Then Err.Raise lngReturn

' Open file in Binary mode
FileNumber = FreeFile
Open Filename For Binary Lock Read Write As #FileNumber

' Determine if file has been modified since last read
If LastModified = FileLastModified(Filename) Then Exit Function

' Read file to get get changes from other users
lngReturn = Read(Filename, FileNumber)
If lngReturn <> 0 Then Err.Raise lngReturn

' Find new RecordNumber
RecordNumber = GetRecordNumberFromID(lngID)

Exit Function

errHandler:
InitializeFileSave = Err

End Function
Public Function Move(Direction As REPOSITION_TYPE, Optional iRecordNumber As Long) As Long

On Error GoTo errHandler

Select Case Direction

  Case Is = MOVE_FIRST
    If RecordCount > 0 Then RecordNumber = 1

  Case Is = MOVE_NEXT
    If RecordNumber < RecordCount Then RecordNumber = RecordNumber + 1

  Case Is = MOVE_PREVIOUS
    If RecordNumber > 1 Then RecordNumber = RecordNumber - 1

  Case Is = MOVE_LAST
    If RecordNumber < RecordCount Then RecordNumber = RecordCount

  Case Is = MOVE_RECORDNUMBER
    If IsMissing(iRecordNumber) Then Exit Function
    If iRecordNumber < 1 Or iRecordNumber > RecordCount Then Err.Raise ERR_RECORD_NOT_FOUND
    RecordNumber = iRecordNumber

End Select

EditMode = EDIT_MODE

Exit Function

errHandler:
Move = Err

End Function
Private Function NewFieldIndex() As Long
Dim i As Long
Dim iHigh As Long

On Error GoTo errHandler

' Returns highest field index + 1.

' Going backwards may minimize swapping.
For i = UBound(m_FieldDefinitions) To LBound(m_FieldDefinitions) Step -1
  If m_FieldDefinitions(i).Index > iHigh Then iHigh = m_FieldDefinitions(i).Index
Next i

NewFieldIndex = iHigh + 1

Exit Function

errHandler:
NewFieldIndex = 1

End Function
Private Function Read(sFilename As String, iFilenum As Integer) As Long
Dim lngReturn As Long

On Error GoTo errHandler

If Not FileExists(sFilename) Then Err.Raise ERR_FILE_NOT_FOUND

' Retrieve header
Get #iFilenum, 1, Header

' Determine if file is correct type and version
lngReturn = ValidHeader
If lngReturn <> 0 Then Err.Raise lngReturn

If Header.RecordCount > 0 Then
  ReDim arr_Record(Header.RecordCount - 1)
End If

' Retrieve field definitions
Get #iFilenum, , m_FieldDefinitions

' Retrieve records
Get #iFilenum, , arr_Record

Exit Function

errHandler:
Read = Err

End Function
Public Property Get Record() As cRecord
Dim i As Long
Dim j As Long
Dim arr() As Variant
Dim m_Record As New cRecord

' After Moving to a Record, set the Record object of the Form using this property

On Error GoTo errHandler

If (RecordNumber = 0) Or (RecordCount = 0) Or (EditMode = ADD_NEW) Then
  Set Record = DefaultRecord
  Set m_Record = Nothing
  Exit Property
End If

With arr_Record(RecordNumber - 1)
  For i = 0 To FieldCount - 1
    If FieldDefinitions(i, fldArray) And IsArray(.Field(i)) Then
      arr = .Field(i)
      If ArrayInitialized(arr) Then
        For j = LBound(arr) To UBound(arr)
          m_Record.AddArray1Value j, arr(j)
        Next j
      End If
    Else
      m_Record.Fields(i) = .Field(i)
    End If
  Next i
End With

Set Record = m_Record
Set m_Record = Nothing

Exit Property

errHandler:
Dim lngErrReturn As Long
lngErrReturn = ErrorHandler(Error, Err, Filename, "cTable.Record (Property Get)")

End Property
Public Property Get RecordCount() As Integer

On Error GoTo errHandler

' RecordCount is base 1
RecordCount = UBound(arr_Record) + 1

Exit Property

errHandler:

' Most probable error is an undimensioned array
RecordCount = 0

End Property
Private Sub RestoreBackupFile()

On Error GoTo errHandler

' If the new file didn't get created then restore it from
' the backup.  Otherwise delete the backup file.
If FileExists(BackupFilename) Then
  If Not FileExists(Filename) Then
    Name BackupFilename As Filename
  Else
    Kill BackupFilename
  End If
End If

fBackup = False

LastModified = FileLastModified(Filename)

Exit Sub

errHandler:
ErrorHandler Error, Err, BackupFilename, "cTable.RestoreBackupFile"
fBackup = False

End Sub
Public Function SaveRecord(Record As cRecord) As Long
Dim lngReturn As Long
Dim iFilenum As Integer

On Error GoTo errHandler

lngReturn = Validate(Record)
If lngReturn <> 0 Then Err.Raise lngReturn

lngReturn = InitializeFileSave(iFilenum)
If lngReturn <> 0 Then Err.Raise lngReturn

If RecordNumber = 0 Then RecordNumber = RecordCount + 1

' Convert record to UDT
lngReturn = SetData(Record)
If lngReturn <> 0 Then Err.Raise lngReturn

' Write file and cleanup functions
lngReturn = FinalizeFileSave(iFilenum)
If lngReturn <> 0 Then Err.Raise lngReturn

EditMode = EDIT_MODE ' Reset the Mode

Exit Function

errHandler:
SaveRecord = Err

Close #iFilenum
If fBackup Then RestoreBackupFile

End Function
Private Function SetData(Record As cRecord) As Long
Dim i As Long
Dim j As Long
Dim lngReturn As Long
Dim arr() As Variant

On Error GoTo errHandler

' Converts Record object fields into UDT format before saving to file

If EditMode = ADD_NEW Then
  ReDim Preserve arr_Record(RecordCount)
  Record.Fields(RECORD_ID) = Header.MaxRecordID + 1
  Header.MaxRecordID = Header.MaxRecordID + 1
Else
  ' Another user deleted record so add to end of file
  If RecordNumber > RecordCount Then
    ReDim Preserve arr_Record(RecordCount)
  End If
End If

Record.Fields(LAST_MODIFIED) = Now()

With arr_Record(RecordNumber - 1)
  For i = 0 To FieldCount - 1
    If FieldDefinitions(i, fldArray) Then
    ' Place array values in temporary array
      If Record.Array1Count > 0 Then
        ReDim arr(Record.Array1Count - 1)
        For j = 0 To Record.Array1Count - 1
          arr(j) = Record.Array1(j)
        Next j
      End If
      ' Assign array to field
      .Field(i) = arr
    Else
      .Field(i) = Record.Fields(i)
    End If
  Next i
End With

Exit Function

errHandler:
SetData = Err

End Function
Public Function Shuffle() As Long
Dim i As Long
Dim lngReturn As Long
Dim iFilenum As Integer
Dim lngRange As Long
Dim lngIndex As Long
Dim t As tRecord

' Returns error

On Error GoTo errHandler

Randomize Timer

lngReturn = InitializeFileSave(iFilenum)
If lngReturn <> 0 Then Err.Raise lngReturn

If RecordCount < 2 Then
  If RecordCount = 0 Then
    RecordNumber = 0
  Else
    RecordNumber = 1
  End If
  Close #iFilenum
  Exit Function
End If

lngRange = UBound(arr_Record) + 1

For i = LBound(arr_Record) To UBound(arr_Record)
  lngIndex = Int(Rnd * lngRange)
  t = arr_Record(i)
  arr_Record(i) = arr_Record(lngIndex)
  arr_Record(lngIndex) = t
Next i

lngReturn = FinalizeFileSave(iFilenum)
If lngReturn <> 0 Then Err.Raise lngReturn

If RecordCount > 0 Then
  RecordNumber = 1
Else
  RecordNumber = 0
End If

Exit Function

errHandler:
Shuffle = Err

Close #iFilenum
If fBackup Then RestoreBackupFile

End Function
Public Function Sort(Field() As Long, SortDirection() As SORT_DIRECTION) As Long
Dim i As Long
Dim lngReturn As Long
Dim iFilenum As Integer

' Sorts Records using LIFO of Field array

On Error GoTo errHandler

lngReturn = InitializeFileSave(iFilenum)
If lngReturn <> 0 Then Err.Raise lngReturn

If RecordCount < 2 Then
  If RecordCount = 0 Then
    RecordNumber = 0
  Else
    RecordNumber = 1
  End If
  Close #iFilenum
  Exit Function
End If

For i = UBound(Field) To LBound(Field) Step -1
  If Not FieldDefinitions(Field(i), fldArray) Then
    lngReturn = SortRecords(Field(i), SortDirection(i))
    If lngReturn <> 0 Then Err.Raise lngReturn
  End If
Next i

lngReturn = FinalizeFileSave(iFilenum)
If lngReturn <> 0 Then Err.Raise lngReturn

If RecordCount > 0 Then
  RecordNumber = 1
Else
  RecordNumber = 0
End If

Exit Function

errHandler:
Sort = Err

Close #iFilenum
If fBackup Then RestoreBackupFile

End Function
Private Function SortRecords(SortField As Long, SortDirection As SORT_DIRECTION)
Dim i As Long
Dim j As Long
Dim lngID As Long
Dim lngInc As Long
Dim tmpRecord As tRecord
Dim fSwap As Boolean

' Attempting to sort an array field will raise an error

On Error GoTo errHandler

' Save ID of current record prior to sorting
If RecordNumber > 0 Then
  lngID = arr_Record(RecordNumber - 1).Field(RECORD_ID)
End If

lngInc = (UBound(arr_Record) + 1) / 2

Do While lngInc > 0
  For i = lngInc To UBound(arr_Record)
    j = i - lngInc
    Do While j >= 0
      If SortDirection = SORT_DESCENDING Then
        If arr_Record(j).Field(SortField) < arr_Record(j + lngInc).Field(SortField) Then
          fSwap = True
        End If
      Else ' Default to Ascending sort order
        If arr_Record(j).Field(SortField) > arr_Record(j + lngInc).Field(SortField) Then
          fSwap = True
        End If
      End If
      If fSwap Then   ' Swap records
        fSwap = False ' Reset flag
        tmpRecord = arr_Record(j)
        arr_Record(j) = arr_Record(j + lngInc)
        arr_Record(j + lngInc) = tmpRecord
        j = j - lngInc
      Else
        j = -1
      End If
    Loop
  Next i
  lngInc = lngInc / 2
Loop

' Use saved ID to find new RecordNumber after sorting
RecordNumber = GetRecordNumberFromID(lngID)

Exit Function

errHandler:
SortRecords = Err

End Function
Private Function Validate(Record As cRecord) As Long
Dim i As Long
Dim lngErr As Long
Dim lngReturn As Long
Dim X As Variant

' Checks if values are the correct data type and
' returns Error
' Final data validation before saving.

On Error GoTo errHandler

lngReturn = HasDuplicateEntry(Record)
If lngReturn <> 0 Then
  Validate = lngReturn
  Exit Function
End If

For i = 0 To FieldCount - 1
  If Not m_FieldDefinitions(i).Required And Trim$(CStr(Record.Fields(i))) = vbNullString Then
    ' Do nothing
  Else
    Select Case m_FieldDefinitions(i).DataType
      ' Try to raise error for data type
      Case Is = dataBoolean
        lngErr = ERR_DATA_BOOLEAN_OUT_OF_RANGE
        X = CBool(Record.Fields(i))

      Case Is = dataByte
        lngErr = ERR_DATA_BYTE_OUT_OF_RANGE
        X = CByte(Record.Fields(i))

      Case Is = dataCurrency
        lngErr = ERR_DATA_CURRENCY_OUT_OF_RANGE
        X = CCur(Record.Fields(i))

      Case Is = dataDate
        lngErr = ERR_DATA_DATE_OUT_OF_RANGE
        X = CDate(Record.Fields(i))

      Case Is = dataDoublePrecision
        lngErr = ERR_DATA_DOUBLE_PRECISION_OUT_OF_RANGE
        X = CDbl(Record.Fields(i))

      Case Is = dataInteger
        lngErr = ERR_DATA_INTEGER_OUT_OF_RANGE
        X = CInt(Record.Fields(i))

      Case Is = dataLongInteger
        lngErr = ERR_DATA_LONG_INTEGER_OUT_OF_RANGE
        X = CLng(Record.Fields(i))

      Case Is = dataSinglePrecision
        lngErr = ERR_DATA_SINGLE_PRECISION_OUT_OF_RANGE
        X = CSng(Record.Fields(i))

      Case Is = dataString
        X = CStr(Record.Fields(i))
        If X = vbNullString Then Err.Raise ERR_REQUIRED_DATA_NOT_ENTERED

    End Select

  End If

Next i

' No error
Validate = 0

Exit Function

errHandler:
If lngErr = 0 Then
  Validate = Err
Else
  Validate = lngErr
End If

End Function
Private Function ValidHeader() As Long

On Error GoTo errHandler

' Ensure that the file being read is of the proper type

With Header
  If .VersionNumber <> VBDB_VERSION_NUMBER Then Err.Raise ERR_BAD_VERSION
  If .VersionString <> VBDB_VERSION_STRING Then Err.Raise ERR_BAD_VERSION
End With

Exit Function

errHandler:
ValidHeader = Err

End Function
Private Function WriteFile(iFilenum As Integer) As Long
Dim lngReturn As Long

On Error GoTo errHandler

' Save header information
CreateHeader
Put #iFilenum, 1, Header

' Save field definitions
Put #iFilenum, , m_FieldDefinitions

' Save records
Put #iFilenum, , arr_Record

Exit Function

errHandler:
WriteFile = Err

End Function
Private Sub Class_Initialize()

EditMode = EDIT_MODE

' Create system fields
CreateSystemFields

' Create default fields
CreateDataFields

End Sub
