VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cTable"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'---------------------------------------------------------------------------------------
' Module    : cTable
' DateTime  : 5/26/2002
' Author    : Paul K. Johnson
' Purpose   : Creates a Binary Access file to save and retrieve record-type data. Uses cRecord class.
'
' This Class does not include multi-user support.
'---------------------------------------------------------------------------------------

Option Explicit

Public RecordNumber As Long
Public SearchField As FIELD_INDEX
Public SearchOperator As COMPARISON_OPERATOR
Public SearchValue As Variant

Private Const CurrentDateTime As Integer = -1
Private nEditMode As Long
Private sFilename As String

' Additional fields can be added to the Header.
' DO NOT REMOVE OR RENAME these four fields!

' VersionString
' VersionNumber
' RecordCount
' MaxRecordID

Private Type tHeader
  VersionString As String
  VersionNumber As String
  RecordCount As Long
  MaxRecordID As Long ' The maximum Index of all records
End Type
Private Header As tHeader

Private Type tFieldDefinition
  Index As Long
  SystemField As Boolean
  FieldName As String
  ArrayField As Boolean
  DataType As Long
  DefaultValue As Variant
  Required As Boolean
  RequireUniqueEntry As Boolean
End Type

Private m_FieldDefinitions(FIELD_INDEX_MAX) As tFieldDefinition

Private Type tRecord
  Field(FIELD_INDEX_MAX) As Variant
End Type

Private arr_Record() As tRecord

Public Function AddRecord() As Long

If EditMode = ADD_NEW Then Exit Function

EditMode = ADD_NEW
RecordNumber = RecordCount + 1

End Function
Private Sub CreateDataFields()

' ***** MODIFY *****

' Before creating fields, modify the FIELD_INDEX Enum in bVBDB.bas
' Then use the values from the Enum as subscripts for your fields

' Note: Assign the Constant CurrentDateTime (-1) to a date
' field to have the value of Now() entered into the value field
' when a new record is created.

' Generic Boolean field
With m_FieldDefinitions(FIELD_01)
  .RequireUniqueEntry = False
  .ArrayField = False
  .DataType = dataBoolean
  .DefaultValue = False
  .FieldName = "Boolean 1"
  .Index = NewFieldIndex
  .Required = False
  .SystemField = False
End With

' Generic Byte field
With m_FieldDefinitions(FIELD_02)
  .RequireUniqueEntry = False
  .ArrayField = False
  .DataType = dataByte
  .DefaultValue = 0
  .FieldName = "Byte 1"
  .Index = NewFieldIndex
  .Required = False
  .SystemField = False
End With

' Generic Date field
With m_FieldDefinitions(FIELD_03)
  .RequireUniqueEntry = False
  .ArrayField = False
  .DataType = dataDate
  .DefaultValue = CurrentDateTime
  .FieldName = "Date 1"
  .Index = NewFieldIndex
  .Required = False
  .SystemField = False
End With

' Generic Double Precision array field
With m_FieldDefinitions(FIELD_04)
  .RequireUniqueEntry = False
  .ArrayField = True
  .DataType = dataDoublePrecision
  .DefaultValue = 0
  .FieldName = "Double Array"
  .Index = NewFieldIndex
  .Required = False
  .SystemField = False
End With

' Generic Integer field
With m_FieldDefinitions(FIELD_05)
  .RequireUniqueEntry = False
  .ArrayField = False
  .DataType = dataInteger
  .DefaultValue = 0
  .FieldName = "Integer 1"
  .Index = NewFieldIndex
  .Required = False
  .SystemField = False
End With

' Generic Long Integer field
With m_FieldDefinitions(FIELD_06)
  .RequireUniqueEntry = False
  .ArrayField = False
  .DataType = dataLongInteger
  .DefaultValue = 0
  .FieldName = "Long 1"
  .Index = NewFieldIndex
  .Required = False
  .SystemField = False
End With

' Generic Single Precision field
With m_FieldDefinitions(FIELD_07)
  .RequireUniqueEntry = False
  .ArrayField = False
  .DataType = dataSinglePrecision
  .DefaultValue = 0
  .FieldName = "Single 1"
  .Index = NewFieldIndex
  .Required = False
  .SystemField = False
End With

With m_FieldDefinitions(FIELD_08)
  .RequireUniqueEntry = False
  .ArrayField = False
  .DataType = dataString
  .DefaultValue = vbNullString
  .FieldName = "String 1"
  .Index = NewFieldIndex
  .Required = False
  .SystemField = False
End With

' ***** END MODIFY *****

End Sub
Private Sub CreateSystemFields()

' ****** DO NOT modify these fields! ******

With m_FieldDefinitions(RECORD_ID)
  .RequireUniqueEntry = False
  .ArrayField = False
  .DataType = dataLongInteger
  .DefaultValue = 0
  .FieldName = "ID"
  .Index = NewFieldIndex
  .Required = True
  .SystemField = True
End With

With m_FieldDefinitions(DATE_CREATED)
  .RequireUniqueEntry = False
  .ArrayField = False
  .DataType = dataDate
  .DefaultValue = CurrentDateTime
  .FieldName = "Date Created"
  .Index = NewFieldIndex
  .Required = True
  .SystemField = True
End With

With m_FieldDefinitions(LAST_MODIFIED)
  .RequireUniqueEntry = False
  .ArrayField = False
  .DataType = dataDate
  .DefaultValue = CurrentDateTime
  .FieldName = "Last Modified"
  .Index = NewFieldIndex
  .Required = True
  .SystemField = True
End With

End Sub
Private Sub CreateHeader()

With Header
  .RecordCount = RecordCount
  .VersionNumber = VBDB_VERSION_NUMBER
  .VersionString = VBDB_VERSION_STRING
End With

End Sub
Private Property Get DefaultRecord() As cRecord
Dim n As Long
Dim m_Record As New cRecord

' Enters default values into fields of Record object

With m_Record
  For n = 0 To FieldCount - 1
    If (m_FieldDefinitions(n).DataType = dataDate) And (m_FieldDefinitions(n).DefaultValue = CurrentDateTime) Then
      .Fields(n) = Now()
    Else
      .Fields(n) = FieldDefinitions(n, fldDefaultValue)
    End If
  Next n
  .Fields(DATE_CREATED) = Now()
  .Fields(LAST_MODIFIED) = .Fields(DATE_CREATED)
  .Fields(RECORD_ID) = 0
End With

Set DefaultRecord = m_Record
Set m_Record = Nothing

End Property
Public Function DeleteRecord() As Long
Dim n As Long
Dim nReturn As Long

On Error GoTo errHandler ' Returns Error Code

If RecordCount = 1 And RecordNumber = 1 Then
  Erase arr_Record
  Exit Function
End If

' Move all records after deleted record forward one position in array
For n = RecordNumber - 1 To UBound(arr_Record) - 1
  arr_Record(n) = arr_Record(n + 1)
Next n

' Redimension the array to remove the last element
ReDim Preserve arr_Record(UBound(arr_Record) - 1)

nReturn = WriteFile
If nReturn <> 0 Then Err.Raise nReturn

' Move to previous record
If RecordNumber > 1 Then
  RecordNumber = RecordNumber - 1
Else
  RecordNumber = 1
End If

' Return 0

Exit Function

errHandler:
DeleteRecord = Err

End Function
Private Property Let EditMode(iMode As EDITING_MODE)

nEditMode = iMode

End Property
Public Property Get EditMode() As EDITING_MODE

EditMode = nEditMode

End Property
Public Property Get FieldCount() As Long
Dim t As tRecord

FieldCount = UBound(t.Field) + 1

End Property
Public Property Get FieldDefinitions(Index As FIELD_INDEX, FieldDefinition As FIELD_DEFINITION) As Variant

With m_FieldDefinitions(Index)

  Select Case FieldDefinition

    Case Is = fldArray
      FieldDefinitions = .ArrayField

    Case Is = fldDataType
      FieldDefinitions = .DataType

    Case Is = fldDefaultValue
      FieldDefinitions = .DefaultValue

    Case Is = fldIndex
      FieldDefinitions = .Index

    Case Is = fldName
      FieldDefinitions = .FieldName

    Case Is = fldRequired
      FieldDefinitions = .Required

    Case Is = fldRequireUnique
      FieldDefinitions = .RequireUniqueEntry

    Case Is = fldSystemField
      FieldDefinitions = .SystemField

  End Select

End With

End Property
Public Property Let FieldDefinitions(Index As FIELD_INDEX, FieldDefinition As FIELD_DEFINITION, Value As Variant)

With m_FieldDefinitions(Index)

  ' Changes to System fields are not allowed
  If .SystemField Then Exit Property

  Select Case FieldDefinition

    Case Is = fldArray
      ' Do nothing

    Case Is = fldDataType
      .DataType = Value

    Case Is = fldDefaultValue
      .DefaultValue = Value

    Case Is = fldName
      .FieldName = Value

    Case Is = fldRequired
      .Required = Value

    Case Is = fldRequireUnique
      .RequireUniqueEntry = Value

    Case Is = fldSystemField
      ' Do nothing

  End Select

End With

End Property
Public Property Get Filename() As String

Filename = sFilename

End Property
Public Property Let Filename(DataFilename As String)
Dim nReturn As Long
Dim sTmp As String

' File is read when Filename property is set.

On Error GoTo errHandler

sTmp = DataFilename

If FileExists(sTmp) Then
  nReturn = Read(sTmp)
  If nReturn <> 0 Then Err.Raise nReturn
Else ' Create a new database file
  sFilename = sTmp
  nReturn = WriteFile
  If nReturn <> 0 Then Err.Raise nReturn
  AddRecord
End If

sFilename = sTmp

Exit Property

errHandler:
Dim nError As Long

nError = ErrorHandler(Error, Err, sFilename, "cTable.Filename(Property Let)")
If nError = vbRetry Then Resume

End Property
Public Function Find(Direction As SEARCH_DIRECTION) As Long
Dim n As Long
Dim nReturn As Long

' Returns Recordnumber or 0 if no record found.

' Does not change current record number.
' Calling procedure must change the recordnumber.

On Error GoTo errHandler

Select Case Direction
  Case Is = FIND_FIRST ' Start at beginning of file and search to the end
    For n = 0 To RecordCount - 1
      nReturn = FindCompare(arr_Record(n).Field(SearchField))
      If nReturn = 0 Then
        Find = n + 1
        Exit Function
      End If
    Next n

  Case Is = FIND_PREVIOUS ' Start at previous record and search to beginning
    For n = RecordNumber - 2 To 0 Step -1
      nReturn = FindCompare(arr_Record(n).Field(SearchField))
      If nReturn = 0 Then
        Find = n + 1
        Exit Function
      End If
    Next n
 
  Case Is = FIND_NEXT ' Start at next record and search to the end
    For n = RecordNumber To RecordCount - 1
      nReturn = FindCompare(arr_Record(n).Field(SearchField))
      If nReturn = 0 Then
        Find = n + 1
        Exit Function
      End If
    Next n

  Case Is = FIND_LAST ' Start at last record and search to the beginning
    For n = RecordCount - 1 To 0 Step -1
      nReturn = FindCompare(arr_Record(n).Field(SearchField))
      If nReturn = 0 Then
        Find = n + 1
        Exit Function
      End If
    Next n

End Select

Err.Raise ERR_RECORD_NOT_FOUND

Exit Function

errHandler:
Find = 0
ErrorHandler Error, Err, SearchValue, "cTable.Find()"

End Function
Private Function FindCompare(v_Field As Variant) As Long

' Returns 0 if record is found

On Error GoTo errHandler

If IsDate(v_Field) And IsDate(SearchValue) Then
  ' Date comparison
  Select Case SearchOperator
    Case EQUAL
      If CDate(v_Field) = CDate(SearchValue) Then Exit Function
    Case NOT_EQUAL
      If CDate(v_Field) <> CDate(SearchValue) Then Exit Function
    Case LESS_THAN
      If CDate(v_Field) < CDate(SearchValue) Then Exit Function
    Case GREATER_THAN
      If CDate(v_Field) > CDate(SearchValue) Then Exit Function
    Case LESS_THAN_OR_EQUAL
      If CDate(v_Field) <= CDate(SearchValue) Then Exit Function
    Case GREATER_THAN_OR_EQUAL
      If CDate(v_Field) >= CDate(SearchValue) Then Exit Function
  End Select

ElseIf CheckNumeric(CStr(v_Field), True) And CheckNumeric(CStr(SearchValue), True) Then
  ' Numeric comparison
  Select Case SearchOperator
    Case EQUAL
      If v_Field = Val(SearchValue) Then Exit Function
    Case NOT_EQUAL
      If v_Field <> Val(SearchValue) Then Exit Function
    Case LESS_THAN
      If v_Field < Val(SearchValue) Then Exit Function
    Case GREATER_THAN
      If v_Field > Val(SearchValue) Then Exit Function
    Case LESS_THAN_OR_EQUAL
      If v_Field <= Val(SearchValue) Then Exit Function
    Case GREATER_THAN_OR_EQUAL
      If v_Field >= Val(SearchValue) Then Exit Function
  End Select

Else ' Default
  ' Text comparison
  Select Case SearchOperator
    Case EQUAL ' Works similar to * LIKE *
      If InStr(1, CStr(v_Field), CStr(SearchValue), vbTextCompare) > 0 Then Exit Function
    Case NOT_EQUAL
      If InStr(1, CStr(v_Field), CStr(SearchValue), vbTextCompare) = 0 Then Exit Function
    Case LESS_THAN
      If StrComp(CStr(SearchValue), CStr(v_Field), vbTextCompare) = -1 Then Exit Function
    Case GREATER_THAN
      If StrComp(CStr(SearchValue), CStr(v_Field), vbTextCompare) = 1 Then Exit Function
    Case LESS_THAN_OR_EQUAL
      If StrComp(CStr(SearchValue), CStr(v_Field), vbTextCompare) <= 0 Then Exit Function
    Case GREATER_THAN_OR_EQUAL
      If StrComp(CStr(SearchValue), CStr(v_Field), vbTextCompare) >= 0 Then Exit Function
  End Select

End If

' Function will not run to here if search was successful
Err.Raise ERR_RECORD_NOT_FOUND

Exit Function

errHandler:
FindCompare = Err

End Function
Private Function HasDuplicateEntry(ByRef Record As cRecord) As Long
Dim n As Long
Dim j As Long

On Error GoTo errHandler ' Returns Error Code

For n = 0 To RecordCount - 1
  If n <> RecordNumber - 1 Then
    For j = 0 To FieldCount - 1
      With m_FieldDefinitions(j)
        If .RequireUniqueEntry And Not .ArrayField Then
          If .DataType = dataString Then
            If StrComp(arr_Record(n).Field(j), Record.Fields(j), vbTextCompare) = 0 Then Err.Raise ERR_DUPLICATE_DATA_NOT_ALLOWED
          Else
            If arr_Record(n).Field(j) = Record.Fields(j) Then Err.Raise ERR_DUPLICATE_DATA_NOT_ALLOWED
          End If
        End If
      End With
    Next j
  End If
Next n

' Return 0

Exit Function

errHandler:

HasDuplicateEntry = Err
ErrorHandler Error, Err, m_FieldDefinitions(j).FieldName, "cTable.HasDuplicateEntry()"

End Function
Public Property Get ID(Index As Long) As Long

ID = arr_Record(Index).Field(RECORD_ID)

End Property
Public Function Move(Direction As REPOSITION_TYPE, Optional RecNum As Long) As Long
Dim nRand As Long

On Error GoTo errHandler ' Returns Error Code

Select Case Direction

  Case Is = MOVE_FIRST
    If RecordCount > 0 Then RecordNumber = 1

  Case Is = MOVE_NEXT
    If RecordNumber < RecordCount Then RecordNumber = RecordNumber + 1

  Case Is = MOVE_PREVIOUS
    If RecordNumber > 1 Then RecordNumber = RecordNumber - 1

  Case Is = MOVE_LAST
    If RecordNumber < RecordCount Then RecordNumber = RecordCount

  Case Is = MOVE_RECORDNUMBER
    If IsMissing(RecNum) Then Exit Function
    If RecNum < 1 Or RecNum > RecordCount Then Err.Raise ERR_RECORD_NOT_FOUND
    RecordNumber = RecNum

  Case Is = MOVE_RANDOM
    If RecordCount <= 1 Then Exit Function

    Randomize Timer

    nRand = RecordNumber

    Do While nRand = RecordNumber
      nRand = CLng((Rnd * (RecordCount - 1)) + 1)
    Loop

    RecordNumber = nRand

End Select

EditMode = EDIT_MODE

' Return 0

Exit Function

errHandler:
Move = Err

End Function
Private Function NewFieldIndex() As Long
Dim n As Long
Dim iHigh As Long

On Error GoTo errHandler

' Returns highest field index + 1.

' Going backwards may minimize swapping.
For n = UBound(m_FieldDefinitions) To LBound(m_FieldDefinitions) Step -1
  If m_FieldDefinitions(n).Index > iHigh Then iHigh = m_FieldDefinitions(n).Index
Next n

NewFieldIndex = iHigh + 1

Exit Function

errHandler:
NewFieldIndex = 1

End Function
Private Function Read(sFilename As String) As Long
Dim nReturn As Long
Dim iFilenum As Integer

On Error GoTo errHandler ' Returns Error Code

iFilenum = FreeFile
Open sFilename For Binary Lock Read Write As #iFilenum

  ' Retrieve header
  Get #iFilenum, 1, Header

  ' Determine if file is correct type and version
  nReturn = ValidHeader
  If nReturn <> 0 Then Err.Raise nReturn

  If Header.RecordCount > 0 Then
    ReDim arr_Record(Header.RecordCount - 1)
  End If

  ' Retrieve field definitions
  Get #iFilenum, , m_FieldDefinitions

  ' Retrieve records
  Get #iFilenum, , arr_Record

Close #iFilenum

' Return 0

Exit Function

errHandler:
Read = Err
Close #iFilenum

End Function
Public Property Get Record() As cRecord
Dim n As Long
Dim j As Long
Dim arr() As Variant
Dim m_Record As New cRecord

' After Moving to a Record, set the Record object of the Form using this property

On Error GoTo errHandler

If (RecordNumber = 0) Or (RecordCount = 0) Or (EditMode = ADD_NEW) Then
  Set Record = DefaultRecord
  Set m_Record = Nothing
  Exit Property
End If

With arr_Record(RecordNumber - 1)
  For n = 0 To FieldCount - 1
    If FieldDefinitions(n, fldArray) And IsArray(.Field(n)) Then
      arr = .Field(n)
      If ArrayInitialized(arr) Then
        For j = LBound(arr) To UBound(arr)
          m_Record.AddArray1Value j, arr(j)
        Next j
      End If
    Else
      m_Record.Fields(n) = .Field(n)
    End If
  Next n
End With

Set Record = m_Record
Set m_Record = Nothing

Exit Property

errHandler:
Dim nErrReturn As Long
nErrReturn = ErrorHandler(Error, Err, Filename, "cTable.Record(Property Get)")

End Property
Public Property Get RecordCount() As Integer

On Error GoTo errHandler

' RecordCount is base 1
RecordCount = UBound(arr_Record) + 1

Exit Property

errHandler:

' Most probable error is an undimensioned array
RecordCount = 0

End Property
Public Function SaveRecord(Record As cRecord) As Long
Dim nReturn As Long

On Error GoTo errHandler ' Returns Error Code

If Not FileExists(Filename) Then Err.Raise APPLICATION_ERROR.ERR_NO_OPEN_DATABASE

nReturn = Validate(Record)
If nReturn <> 0 Then Err.Raise nReturn

nReturn = SetData(Record)
If nReturn <> 0 Then Err.Raise nReturn

nReturn = WriteFile
If nReturn <> 0 Then Err.Raise nReturn

EditMode = EDIT_MODE ' Reset the Mode

' Return 0

Exit Function

errHandler:
SaveRecord = Err

End Function
Private Function SetData(Record As cRecord) As Long
Dim n As Long
Dim j As Long
Dim arr() As Variant

On Error GoTo errHandler ' Returns Error Code

' Converts Record object fields into UDT format before saving to file

If EditMode = ADD_NEW Then
  ReDim Preserve arr_Record(RecordCount)
  Record.Fields(RECORD_ID) = Header.MaxRecordID + 1
  Header.MaxRecordID = Header.MaxRecordID + 1
End If

Record.Fields(LAST_MODIFIED) = Now()

With arr_Record(RecordNumber - 1)
  For n = 0 To FieldCount - 1
    If FieldDefinitions(n, fldArray) Then
    ' Place array values in temporary array
      If Record.Array1Count > 0 Then
        ReDim arr(Record.Array1Count - 1)
        For j = 0 To Record.Array1Count - 1
          arr(j) = Record.Array1(j)
        Next j
      End If
      ' Assign array to field
      .Field(n) = arr
    Else
      .Field(n) = Record.Fields(n)
    End If
  Next n
End With

' Return 0

Exit Function

errHandler:
SetData = Err

End Function
Public Function Shuffle() As Long
Dim n As Long
Dim nID As Long
Dim nRange As Long
Dim nIndex As Long
Dim t As tRecord

On Error GoTo errHandler ' Returns Error Code

Randomize Timer

' Save ID of current record prior to shuffling
If RecordNumber > 0 Then
  nID = arr_Record(RecordNumber - 1).Field(RECORD_ID)
End If

nRange = UBound(arr_Record) + 1

For n = LBound(arr_Record) To UBound(arr_Record)
  nIndex = Int(Rnd * nRange)
  t = arr_Record(n)
  arr_Record(n) = arr_Record(nIndex)
  arr_Record(nIndex) = t
Next n

WriteFile

' Use saved ID to find new RecordNumber after shuffling
For n = LBound(arr_Record) To UBound(arr_Record)
  If nID = arr_Record(n).Field(RECORD_ID) Then
    RecordNumber = n + 1
    Exit For
  End If
Next n

' Return 0

Exit Function

errHandler:
Shuffle = Err
LogError Error, Err, vbNullString, "cTable.Shuffle()"

End Function
Public Function Sort(Field() As Long, SortDirection() As SORT_DIRECTION) As Long
Dim n As Long
Dim nReturn As Long

On Error GoTo errHandler ' Returns Error Code

' Sort using LIFO

For n = UBound(Field) To LBound(Field) Step -1
  If Not FieldDefinitions(Field(n), fldArray) Then
    nReturn = SortRecords(Field(n), SortDirection(n))
    If nReturn <> 0 Then Err.Raise nReturn
  End If
Next n

WriteFile

' Return 0

Exit Function

errHandler:
Sort = Err

End Function
Private Function SortRecords(SortField As Long, SortDirection As SORT_DIRECTION)
Dim n As Long
Dim j As Long
Dim nID As Long
Dim nInc As Long
Dim tmpRecord As tRecord
Dim fSwap As Boolean

' Attempting to sort an array field will raise an error

On Error GoTo errHandler ' Returns Error Code

' Save ID of current record prior to sorting
If RecordNumber > 0 Then
  nID = arr_Record(RecordNumber - 1).Field(RECORD_ID)
End If

nInc = (UBound(arr_Record) + 1) / 2

Do While nInc > 0
  For n = nInc To UBound(arr_Record)
    j = n - nInc
    Do While j >= 0
      If SortDirection = SORT_DESCENDING Then
        If arr_Record(j).Field(SortField) < arr_Record(j + nInc).Field(SortField) Then
          fSwap = True
        End If
      Else ' Default to Ascending sort order
        If arr_Record(j).Field(SortField) > arr_Record(j + nInc).Field(SortField) Then
          fSwap = True
        End If
      End If
      If fSwap Then   ' Swap records
        fSwap = False ' Reset flag
        tmpRecord = arr_Record(j)
        arr_Record(j) = arr_Record(j + nInc)
        arr_Record(j + nInc) = tmpRecord
        j = j - nInc
      Else
        j = -1
      End If
    Loop
  Next n
  nInc = nInc / 2
Loop

' Use saved ID to find new RecordNumber after sorting
For n = LBound(arr_Record) To UBound(arr_Record)
  If nID = arr_Record(n).Field(RECORD_ID) Then
    RecordNumber = n + 1
    Exit For
  End If
Next n

' Return 0

Exit Function

errHandler:
SortRecords = Err

End Function
Private Function Validate(Record As cRecord) As Long
Dim n As Long
Dim nErr As Long
Dim nReturn As Long
Dim X As Variant

' Checks if values are the correct data type.
' Final data validation before saving.

On Error GoTo errHandler ' Returns Error Code

nReturn = HasDuplicateEntry(Record)
If nReturn <> 0 Then
  Validate = nReturn
  Exit Function
End If

For n = 0 To FieldCount - 1
  If Not m_FieldDefinitions(n).Required And Trim$(CStr(Record.Fields(n))) = vbNullString Then
    ' Do nothing
  Else
    Select Case m_FieldDefinitions(n).DataType
      ' Try to raise error for data type
      Case Is = dataBoolean
        nErr = ERR_DATA_BOOLEAN_OUT_OF_RANGE
        X = CBool(Record.Fields(n))

      Case Is = dataByte
        nErr = ERR_DATA_BYTE_OUT_OF_RANGE
        X = CByte(Record.Fields(n))

      Case Is = dataCurrency
        nErr = ERR_DATA_CURRENCY_OUT_OF_RANGE
        X = CCur(Record.Fields(n))

      Case Is = dataDate
        nErr = ERR_DATA_DATE_OUT_OF_RANGE
        X = CDate(Record.Fields(n))

      Case Is = dataDoublePrecision
        nErr = ERR_DATA_DOUBLE_PRECISION_OUT_OF_RANGE
        X = CDbl(Record.Fields(n))

      Case Is = dataInteger
        nErr = ERR_DATA_INTEGER_OUT_OF_RANGE
        X = CInt(Record.Fields(n))

      Case Is = dataLongInteger
        nErr = ERR_DATA_LONG_INTEGER_OUT_OF_RANGE
        X = CLng(Record.Fields(n))

      Case Is = dataSinglePrecision
        nErr = ERR_DATA_SINGLE_PRECISION_OUT_OF_RANGE
        X = CSng(Record.Fields(n))

      Case Is = dataString
        X = CStr(Record.Fields(n))
        If X = vbNullString Then Err.Raise ERR_REQUIRED_DATA_NOT_ENTERED

    End Select

  End If

Next n

' No error
Validate = 0

Exit Function

errHandler:
If nErr = 0 Then
  Validate = Err
Else
  Validate = nErr
End If

End Function
Private Function ValidHeader() As Long

On Error GoTo errHandler ' Returns Error Code

' Ensure that the file being read is of the proper type

With Header
  If .VersionNumber <> VBDB_VERSION_NUMBER Then Err.Raise ERR_BAD_VERSION
  If .VersionString <> VBDB_VERSION_STRING Then Err.Raise ERR_BAD_VERSION
End With

' Return 0

Exit Function

errHandler:
ValidHeader = Err

End Function
Private Function WriteFile() As Long
Dim iFilenum As Integer
Dim sTempFile As String

On Error GoTo errHandler ' Returns Error Code

' Create a temporary file
sTempFile = GetTempFilename

' Save data to temporary file
iFilenum = FreeFile
Open sTempFile For Binary Lock Read Write As #iFilenum

  CreateHeader
  ' Save header information
  Put #iFilenum, 1, Header

  ' Save field definitions
  Put #iFilenum, , m_FieldDefinitions

  ' Save records
  Put #iFilenum, , arr_Record

Close #iFilenum

If FileExists(Filename) Then
  If Not ReplaceFile(Filename, sTempFile) Then              ' Save operation failed
    If FileExists(Filename) And FileExists(sTempFile) Then
      Kill Filename                                         ' Delete existing file
      Name sTempFile As Filename                            ' Rename temp file to existing file
    End If
  End If
Else
  Name sTempFile As Filename
End If

' Return 0

Exit Function

errHandler:
WriteFile = Err

Close #iFilenum

End Function
Private Sub Class_Initialize()

EditMode = EDIT_MODE

' Create system fields
CreateSystemFields

' Create default fields
CreateDataFields

End Sub
